@@ -48,32 +48,30 @@ SearchResults <- R6::R6Class("SearchResults",
     #' @param force Optional; forces the download even if the file already exists in the specified output directory.
     #' @param prompt Optional; enables all user prompts for decisions during file downloads. Defaults to true.
     #' @return Nothing returned but downloaded files are saved at the specified location.
+    #' @importFrom progress progress_bar
     #' @export
     download = function(output_dir, selected_indexes, stop_at_failure = TRUE, force = FALSE, prompt = TRUE) {
       if (self$total_count == 0 || (prompt && !private$prompt_user_confirmation(self$total_size))) {
         return(NULL)
       }
 
-      message("[Download] Start")
-
       if (!dir.exists(output_dir)) {
         dir.create(output_dir)
       }
 
-      resources_to_download <- self$results
+      resources_to_download <- if (missing(selected_indexes)) self$results else self$results[selected_indexes]
+
+      proggress_bar <- progress::progress_bar$new(
+        format = "[Download] :current/:total (:percent) :elapsed",
+        total = length(resources_to_download),
+        clear = FALSE,
+        width = 60
+      )
 
-      # resources_to_download <- if (missing(selected_indexes)) self$results else self$results[selected_indexes]
       i <- 0
-      should_break <- FALSE
       for (r in resources_to_download) {
-        if (should_break) {
-          break
-        }
-
         i <- i + 1
 
-        message(paste0("[Download] Downloading file ", i, "/", length(resources_to_download)))
-
         tryCatch(
           {
             download_id <- private$get_download_id(r)
@@ -81,14 +79,28 @@ SearchResults <- R6::R6Class("SearchResults",
             if (is_ready) {
               private$download_resource(download_id, output_dir, force)
             }
+            proggress_bar$tick()
           },
           error = function(err) {
-            warning("Error during the downloading:", err, sep = "\n")
-            should_break <<- stop_at_failure
+            if (stop_at_failure) {
+              proggress_bar$terminate()
+              stop(conditionMessage(err))
+            } else {
+              err_msg <- conditionMessage(err)
+              message(err_msg)
+
+              # Special handling for HTTP 429
+              if (grepl("\\[HTTP 429\\]", err_msg)) {
+                proggress_bar$terminate()
+                stop(sprintf("Operation halted due to rate limit (HTTP 429). Please slow down your requests."))
+              }
+
+              proggress_bar$message(conditionMessage(err))
+              proggress_bar$tick()
+            }
           }
         )
       }
-      message("[Download] DONE")
     }
   ),
   private = list(
@@ -134,7 +146,7 @@ SearchResults <- R6::R6Class("SearchResults",
 
       # Check if the file already exists and force flag is FALSE
       if (file.exists(file_path) && !force) {
-        warning("File already exists:", file_path, "- Skipping download.\n")
+        # warning("File already exists:", file_path, "- Skipping download.\n")
         return(NA)
       }
 
@@ -143,7 +155,12 @@ SearchResults <- R6::R6Class("SearchResults",
 
       resp <- private$client$send_request(req, TRUE)
       if (resp$status_code != 200) {
-        stop(paste("Couldn't download: ", url))
+        stop(sprintf(
+          "Failed to download data.\nURL: %s\nStatus code: %s\nReason: %s",
+          url,
+          resp$status_code,
+          resp$status_text %||% "Unknown"
+        ))
       }
 
       writeBin(httr2::resp_body_raw(resp), file_path)
@@ -152,7 +169,7 @@ SearchResults <- R6::R6Class("SearchResults",
     prompt_user_confirmation = function(total_size) {
       if (total_size >= private$LARGE_DOWNLOAD_SIZE) {
         repeat {
-          message("The total size is ", humanize::natural_size(total_size), ". Do you want to proceed? (Y/N): ")
+          message("The total size is ", scales::label_bytes()(total_size), ". Do you want to proceed? (Y/N): ")
           answer <- tolower(readLines(n = 1))
           if (answer %in% c("y", "n")) {
             return(answer == "y")
@@ -171,7 +188,12 @@ SearchResults <- R6::R6Class("SearchResults",
 
       resp <- private$client$send_request(req, TRUE)
       if (resp$status_code != 200) {
-        stop(paste("Couldn't download: ", url))
+        stop(sprintf(
+          "Failed to download data.\nURL: %s\nStatus code: %s\nReason: %s",
+          url,
+          resp$status_code,
+          resp$status_text %||% "Unknown"
+        ))
       }
 
       # Extract the file name from the Content-Disposition header
